@@ -1,7 +1,6 @@
-#############################################################################################################
+################################################################################
 # Author :
-#   Florian Rohart, The University of Queensland, The University of Queensland Diamantina Institute, Translational Research Institute, Brisbane, QLD
-#   Kim-Anh Le Cao, The University of Queensland, The University of Queensland Diamantina Institute, Translational Research Institute, Brisbane, QLD
+#   Florian Rohart,
 #
 # created: 04-07-2015
 # last modified: 22-04-2016
@@ -21,27 +20,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-#############################################################################################################
+################################################################################
 
 
-# ========================================================================================================
-# perform a Leave-one-out cross validation on the study to tune the number of variables (keepX) to keep in a mint.splsda analysis
-# ========================================================================================================
+# ==============================================================================
+# perform a Leave-one-out cross validation on the study to tune the number of
+# variables (keepX) to keep in a mint.splsda analysis
+# ==============================================================================
 
 
-LOGOCV = function(X,
-Y,
-ncomp,
-study,
-choice.keepX = NULL,
-test.keepX = c(5, 10, 15),
-dist = "max.dist",
-measure = c("BER"), # one of c("overall","BER")
-auc = auc,
-max.iter = 100,
-progressBar = TRUE,
-near.zero.var = FALSE,
-scale)
+LOGOCV <- function(X,
+                  Y,
+                  ncomp,
+                  study,
+                  choice.keepX = NULL,
+                  test.keepX = c(5, 10, 15),
+                  dist = "max.dist",
+                  measure = c("BER"), # one of c("overall","BER")
+                  auc = auc,
+                  max.iter = 100,
+                  progressBar = FALSE,
+                  near.zero.var = FALSE,
+                  scale)
 {
     # X input
     # Y factor input
@@ -54,25 +54,25 @@ scale)
     
     
     if(missing(X))
-    stop("missing X")
+        stop("missing X")
     
     if(missing(Y))
-    stop("missing Y")
+        stop("missing Y")
     
     if(missing(study))
-    stop("missing study")
+        stop("missing study")
     
     if(missing(ncomp))
-    ncomp = 1
+        ncomp = 1
     
     if(missing(scale))
-    scale = FALSE
+        scale = FALSE
     
     if(missing(dist))
-    dist = "max.dist"
+        dist = "max.dist"
     
     if(length(Y) != nrow(X))
-    stop("X and Y have to be of same length")
+        stop("X and Y have to be of same length")
     
     #-- set up a progress bar --#
     if (progressBar ==  TRUE)
@@ -86,11 +86,13 @@ scale)
     M = nlevels(study)
     names.study = levels(study)
     features = NULL
-    prediction.comp = array(0, c(nrow(X), nlevels(Y), length(test.keepX)), dimnames = list(rownames(X), levels(Y), test.keepX))
+    prediction.comp = array(0, c(nrow(X), nlevels(Y), length(test.keepX)),
+                            dimnames = list(rownames(X), levels(Y), test.keepX))
     
     class.comp = list()
     for(ijk in dist)
-    class.comp[[ijk]] = matrix(0, nrow = nrow(X), ncol = length(test.keepX))# prediction of all samples for each test.keepX and  nrep at comp fixed
+        class.comp[[ijk]] = matrix(0, nrow = nrow(X), ncol = length(test.keepX))
+    # prediction of all samples for each test.keepX and  nrep at comp fixed
     
     PRED=INDICE = matrix(0, nrow = length(test.keepX), ncol = M)
     rownames(PRED) = rownames(INDICE) = test.keepX
@@ -99,14 +101,16 @@ scale)
     for (study_i in 1:M) #LOO on the study factor
     {
         if (progressBar ==  TRUE)
-        setTxtProgressBar(pb, (study_i-1)/M)
+            setTxtProgressBar(pb, (study_i-1)/M)
         
         omit = which(study %in% names.study[study_i])
         X.train = X[-omit,]
         Y.train = factor(Y[-omit])
         study.learn.CV = factor(as.character(study[-omit]))
         
-        X.test = X[omit, , drop = FALSE] #note: drop is useless as there should always be more than a single sample in a study
+        X.test = X[omit, , drop = FALSE]
+        #note: drop is useless as there should always be more than a single
+        # sample in a study
         Y.test = Y[omit]
         study.test.CV = factor(as.character(study[omit]))
         
@@ -128,31 +132,41 @@ scale)
         for (i in 1:length(test.keepX))
         {
             if (progressBar ==  TRUE)
-            setTxtProgressBar(pb, (study_i-1)/M + (i-1)/length(test.keepX)/M)
+                setTxtProgressBar(pb, (study_i-1)/M + (i-1)/length(test.keepX)/M)
+            
+            object.res = mint.splsda(X.train, Y.train,
+                                                      study = study.learn.CV, ncomp = ncomp, keepX =
+                                                          c(choice.keepX, test.keepX[i]),
+                                                      scale = scale, max.iter = max.iter)
             
-            object.res = suppressWarnings(mint.splsda(X.train, Y.train, study = study.learn.CV, ncomp = ncomp, keepX = c(choice.keepX, test.keepX[i]),
-            scale = scale, mode = "regression", max.iter = max.iter)) # suppress NA warnings from explained_variance
-
             # record selected features
-            if (length(test.keepX) ==  1) # only done if only one test.keepX as not used if more so far
-            features = c(features, selectVar(object.res, comp = ncomp)$name)
+            if (length(test.keepX) ==  1)
+                # only done if only one test.keepX as not used if more so far
+                features = c(features, selectVar(object.res, comp = ncomp)$name)
             
-            test.predict.sw <- predict.spls(object.res, newdata = X.test, method = dist, study.test = study.test.CV)
-            # Y.train can be missing factors, so the prediction 'test.predict.sw' might be missing factors compared to the full prediction.comp
-            prediction.comp[omit, match(levels(Y.train),levels(Y)) , i] =  test.predict.sw$predict[, , ncomp]
+            test.predict.sw <- predict.mixo_spls(object.res, newdata = X.test,
+                                                 method = dist, study.test = study.test.CV)
+            # Y.train can be missing factors, so the prediction
+            # 'test.predict.sw' might be missing factors compared to the
+            # full prediction.comp
+            prediction.comp[omit, match(levels(Y.train),levels(Y)) , i] =
+                test.predict.sw$predict[, , ncomp]
             
             for(ijk in dist)
-            class.comp[[ijk]][omit,i] =  test.predict.sw$class[[ijk]][, ncomp] #levels(Y)[test.predict.sw$class[[ijk]][, ncomp]]
+                class.comp[[ijk]][omit,i] =  test.predict.sw$class[[ijk]][, ncomp]
+            #levels(Y)[test.predict.sw$class[[ijk]][, ncomp]]
         }#end test.keepX
         
         if (progressBar ==  TRUE)
-        setTxtProgressBar(pb, (study_i)/M)
+            setTxtProgressBar(pb, (study_i)/M)
         
     } # end study_i 1:M (M folds)
     
     result = list()
     
-    auc.mean = error.mean = error.sd = error.per.class.keepX.opt.comp = keepX.opt = test.keepX.out = choice.keepX.out = error.per.study.keepX.opt = list()
+    auc.mean = error.mean = error.sd = error.per.class.keepX.opt.comp =
+        keepX.opt = test.keepX.out = choice.keepX.out =
+        error.per.study.keepX.opt = list()
     
     if(auc)
     {
@@ -176,16 +190,21 @@ scale)
             rownames(class.comp[[ijk]]) = rownames(X)
             colnames(class.comp[[ijk]]) = paste0("test.keepX.",test.keepX)
             
-            #finding the best keepX depending on the error measure: overall or BER
-            # classification error for each nrep and each test.keepX: summing over all samples
+            #finding the best keepX depending on the error measure:
+            #   overall or BER
+            # classification error for each nrep and each test.keepX:
+            #   summing over all samples
             error = apply(class.comp[[ijk]], 2, function(x)
             {
                 sum(as.character(Y) != x)
             })
             
-            # we divide the error by the number of samples and choose the minimum error
+            # we divide the error by the number of samples and choose the
+            #   minimum error
             error.mean[[ijk]] = error/length(Y)
-            keepX.opt[[ijk]] = which(error.mean[[ijk]] ==  min(error.mean[[ijk]]))[1] # chose the lowest keepX if several minimum
+            keepX.opt[[ijk]] = which(error.mean[[ijk]] ==
+                                         min(error.mean[[ijk]]))[1]
+            # chose the lowest keepX if several minimum
             
             # overall error per study
             temp = matrix(0, ncol = length(test.keepX), nrow = nlevels(study))
@@ -200,21 +219,24 @@ scale)
             error.per.study.keepX.opt[[ijk]] = temp[,keepX.opt[[ijk]]]
             
             # confusion matrix for keepX.opt
-            error.per.class.keepX.opt.comp[[ijk]] = apply(class.comp[[ijk]][, keepX.opt[[ijk]], drop = FALSE], 2, function(x)
-            {
-                conf = get.confusion_matrix(truth = factor(Y), predicted = x)
-                out = (apply(conf, 1, sum) - diag(conf)) / summary(Y)
-            })
+            error.per.class.keepX.opt.comp[[ijk]] =
+                apply(class.comp[[ijk]][, keepX.opt[[ijk]], drop = FALSE], 2,
+                      function(x)
+                      {
+                          conf = get.confusion_matrix(truth = factor(Y), predicted = x)
+                          out = (apply(conf, 1, sum) - diag(conf)) / summary(Y)
+                      })
             
             rownames(error.per.class.keepX.opt.comp[[ijk]]) = levels(Y)
             
             
             test.keepX.out[[ijk]] = test.keepX[keepX.opt[[ijk]]]
-
+            
             choice.keepX.out[[ijk]] = c(choice.keepX, test.keepX.out)
             
             result$"overall"$error.rate.mean = error.mean
-            result$"overall"$error.per.study.keepX.opt = error.per.study.keepX.opt
+            result$"overall"$error.per.study.keepX.opt =
+                error.per.study.keepX.opt
             result$"overall"$confusion = error.per.class.keepX.opt.comp
             result$"overall"$keepX.opt = test.keepX.out
         }
@@ -227,38 +249,47 @@ scale)
             rownames(class.comp[[ijk]]) = rownames(X)
             colnames(class.comp[[ijk]]) = paste0("test.keepX.",test.keepX)
             
-            # we calculate a BER for each study and each test.keepX, then average over the study factor
+            # we calculate a BER for each study and each test.keepX,
+            # then average over the study factor
             error = matrix(0, ncol = length(test.keepX), nrow = nlevels(study))
             for (study_i in 1:M) #LOO on the study factor
             {
                 omit = which(study %in% names.study[study_i])
                 error[study_i,] = apply(class.comp[[ijk]][omit,], 2, function(x)
                 {
-                    conf = get.confusion_matrix(truth = factor(Y[omit]), all.levels = levels(factor(Y)), predicted = x)
+                    conf = get.confusion_matrix(truth = factor(Y[omit]),
+                                                all.levels = levels(factor(Y)), predicted = x)
                     get.BER(conf)
                 })
             }
             
-            # average BER over the study
-            error.mean[[ijk]] = apply(error, 2, mean)
-            keepX.opt[[ijk]] = which(error.mean[[ijk]] ==  min(error.mean[[ijk]]))[1]
+            # weighted average BER over the studies
+            error.mean[[ijk]] = apply(error, 2, function(x) {
+                sum(x * table(study)/length(study))
+            })
+            
+            
+            keepX.opt[[ijk]] =
+                which(error.mean[[ijk]] ==  min(error.mean[[ijk]]))[1]
             
             # error per study
             error.per.study.keepX.opt[[ijk]] = error[,keepX.opt[[ijk]]]
             
             # confusion matrix for keepX.opt
-            error.per.class.keepX.opt.comp[[ijk]] = apply(class.comp[[ijk]][, keepX.opt[[ijk]], drop = FALSE], 2, function(x)
-            {
-                conf = get.confusion_matrix(truth = factor(Y), predicted = x)
-                out = (apply(conf, 1, sum) - diag(conf)) / summary(Y)
-            })
+            error.per.class.keepX.opt.comp[[ijk]] =
+                apply(class.comp[[ijk]][, keepX.opt[[ijk]], drop = FALSE], 2,
+                      function(x)
+                      {
+                          conf = get.confusion_matrix(truth = factor(Y), predicted = x)
+                          out = (apply(conf, 1, sum) - diag(conf)) / summary(Y)
+                      })
             
             rownames(error.per.class.keepX.opt.comp[[ijk]]) = levels(Y)
             
             
             test.keepX.out[[ijk]] = test.keepX[keepX.opt[[ijk]]]
             choice.keepX.out[[ijk]] = c(choice.keepX, test.keepX.out)
-
+            
             result$"BER"$error.rate.mean = error.mean
             result$"BER"$error.per.study.keepX.opt = error.per.study.keepX.opt
             result$"BER"$confusion = error.per.class.keepX.opt.comp
@@ -266,11 +297,12 @@ scale)
         }
         
     }
-
+    
     result$prediction.comp = prediction.comp
     result$class.comp = class.comp
     result$auc = auc.mean
-    result$features$stable = sort(table(as.factor(features))/M, decreasing = TRUE)
+    result$features$stable =
+        sort(table(as.factor(features))/M, decreasing = TRUE)
     return(result)
     
 }#end function
