@@ -1,7 +1,7 @@
-#############################################################################################################
+################################################################################
 # Author:
 #   Francois Bartolo,
-#   Florian Rohart, The University of Queensland, The University of Queensland Diamantina Institute, Translational Research Institute, Brisbane, QLD
+#   Florian Rohart,
 #
 # created: 29-08-2016
 # last modified: 30-08-2016
@@ -21,14 +21,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-#############################################################################################################
+################################################################################
 
-# --------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Internal helpers functions to run "plot.perf" functions
-# --------------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 
-internal_graphic.perf<- function (error.rate, error.rate.sd, overlay, type, measure, dist, legend.position, xlab, ylab, color, ...)
+internal_graphic.perf <-
+    function (error.rate,
+              error.rate.sd,
+              overlay,
+              type,
+              measure,
+              dist,
+              legend.position,
+              xlab,
+              ylab,
+              color,
+              ylim = NULL,
+              ...
+    )
 {
     # error.rate is a list [[measure]]
     # error.rate[[measure]] is a matrix of dist columns and ncomp rows
@@ -52,41 +65,55 @@ internal_graphic.perf<- function (error.rate, error.rate.sd, overlay, type, meas
             colnames(temp) = paste(mea, colnames(temp),sep="_")
             error.rate.sd.concat = cbind(error.rate.sd.concat, temp)
         }
-        ylim = range(c(error.rate.concat + error.rate.sd.concat), c(error.rate.concat - error.rate.sd.concat))
+        if(is.null(ylim))
+            ylim = range(c(error.rate.concat + error.rate.sd.concat),
+                         c(error.rate.concat - error.rate.sd.concat))
         
     } else {
         error.rate.sd.concat = NULL
-        ylim = range(error.rate.concat)
+        if(is.null(ylim))
+            ylim = range(error.rate.concat)
         
     }
     
-    #extract component numbers from rownames(error.rate.concat)
+    ## extract component numbers from rownames(error.rate.concat)
+    
+    rownames(error.rate.concat) = gsub("comp","",rownames(error.rate.concat))
     
-    rownames(error.rate.concat) = sapply(strsplit(rownames(error.rate.concat)," "), function(x){x[2]})
     component = as.numeric(rownames(error.rate.concat))
     
     if(overlay == "all")
     {
-        out<-matplot(component, error.rate.concat, type = type, lty = rep(c(1:length(measure)), each = length(dist)), col = rep(color, length(measure)),
-        lwd = 2, xlab = xlab, ylab = ylab, xaxt="n", ylim = ylim)
+        out<-matplot(component, error.rate.concat, type = type, lty =
+                         rep(c(1:length(measure)), each = length(dist)),
+                     col = rep(color, length(measure)),
+                     lwd = 2, xlab = xlab, ylab = ylab, xaxt="n", ylim = ylim)
         
         axis(1, rownames(error.rate.concat))#, rownames(error.rate.concat))
         
         if(legend.position == "vertical")
         {
-            legend('topright', legend = c(measure, dist), lty = c(1:length(measure), rep(NA, length(dist))),
-            pch = c(rep(NA, length(measure)), rep(16, length(dist))), col = c(rep('black', length(measure)), color), ncol = 1, lwd = 2)
+            legend('topright', legend = c(measure, dist),
+                   lty = c(1:length(measure), rep(NA, length(dist))),
+                   pch = c(rep(NA, length(measure)), rep(16, length(dist))),
+                   col = c(rep('black', length(measure)), color), ncol = 1, lwd = 2)
         } else if(legend.position == "horizontal") {
-            legend('topright', legend = c(measure, "" , dist), lty = c(1:length(measure), rep(NA, (length(dist)+1))),
-            pch = c(rep(NA, (length(measure)+1)), rep(16, length(dist))), col = c(rep('black', length(measure)), NA, color), ncol = 2, lwd = 2)
+            legend('topright', legend = c(measure, "" , dist),
+                   lty = c(1:length(measure), rep(NA, (length(dist)+1))),
+                   pch = c(rep(NA, (length(measure)+1)), rep(16, length(dist))),
+                   col = c(rep('black', length(measure)), NA, color), ncol = 2,
+                   lwd = 2)
             
         }
         if(!is.null(error.rate.sd.concat))
         {
-            col = matrix(rep(color, length(measure)), nrow = nrow(error.rate.concat), ncol = length(measure)*length(dist), byrow=TRUE)
+            col = matrix(rep(color, length(measure)),
+                         nrow = nrow(error.rate.concat), ncol = length(measure)*length(dist),
+                         byrow=TRUE)
             
             for(j in 1:ncol(error.rate.concat))
-            plot_error_bar(x = component, y = error.rate.concat[, j], uiw=error.rate.sd.concat[, j], add=T, col = col[, j])
+                plot_error_bar(x = component, y = error.rate.concat[, j],
+                               uiw=error.rate.sd.concat[, j], add=TRUE, col = col[, j])
         }
     } else if(overlay == "measure") {
         
@@ -94,26 +121,32 @@ internal_graphic.perf<- function (error.rate, error.rate.sd, overlay, type, meas
         par(mfrow=c(1, length(dist)))
         for(di in dist)
         {
-            new_mat.error = error.rate.concat[, grep(di, colnames(error.rate.concat)), drop = FALSE]
+            new_mat.error =
+                error.rate.concat[, grep(di, colnames(error.rate.concat)),
+                                  drop = FALSE]
             
             out<-matplot(new_mat.error, type = type, lty = c(1:length(measure)),
-            col = rep(color[which(di == dist)]),
-            lwd = 2, xlab = xlab, ylab = ylab, xaxt="n", ylim = ylim)
+                         col = rep(color[which(di == dist)]),
+                         lwd = 2, xlab = xlab, ylab = ylab, xaxt="n", ylim = ylim)
             
             axis(1, rownames(error.rate.concat))#, rownames(error.rate.concat))
             #axis(2)
             
             if(legend.position == "vertical")
             {
-                legend('topright', legend = measure, lty = 1:length(measure), col = rep(color[which(di == dist)]), ncol = 1, lwd = 2)
+                legend('topright', legend = measure, lty = 1:length(measure),
+                       col = rep(color[which(di == dist)]), ncol = 1, lwd = 2)
             } else if(legend.position == "horizontal") {
-                legend('topright', legend = c(measure), lty = 1:length(measure), col = rep(color[which(di == dist)]), ncol = 2, lwd = 2)
+                legend('topright', legend = c(measure), lty = 1:length(measure),
+                       col = rep(color[which(di == dist)]), ncol = 2, lwd = 2)
             }
             if(!is.null(error.rate.sd.concat))
             {
-                new_sd.error = error.rate.sd.concat[, grep(di, colnames(error.rate.sd.concat)), drop = FALSE]
+                new_sd.error = error.rate.sd.concat[, grep(di,
+                                                           colnames(error.rate.sd.concat)), drop = FALSE]
                 for(j in 1:ncol(new_mat.error))
-                plot_error_bar(x = component, y = new_mat.error[, j], uiw=new_sd.error[, j], add=T, col = color[which(di == dist)])
+                    plot_error_bar(x = component, y = new_mat.error[, j],
+                                   uiw=new_sd.error[, j], add=TRUE, col = color[which(di == dist)])
             }
             #add title for each subgraph
             title(di, line = 1)
@@ -126,28 +159,34 @@ internal_graphic.perf<- function (error.rate, error.rate.sd, overlay, type, meas
         par(mfrow=c(1, length(measure)))
         for(mea in measure)
         {
-            new_mat.error=error.rate.concat[, grep(mea, colnames(error.rate.concat)), drop = FALSE]
+            new_mat.error=error.rate.concat[, grep(mea,
+                                                   colnames(error.rate.concat)), drop = FALSE]
             
             out<-matplot(new_mat.error, type = type, lty = c(1:length(dist)),
-            col = color,
-            lwd = 2, xlab = xlab, ylab = ylab, xaxt="n", ylim = ylim)
+                         col = color,
+                         lwd = 2, xlab = xlab, ylab = ylab, xaxt="n", ylim = ylim)
             
             axis(1, rownames(error.rate.concat))#, rownames(error.rate.concat))
             #axis(2)
             
             if(legend.position == "vertical")
             {
-                legend('topright', legend = dist, lty = 1:length(dist), col = color, ncol = 1, lwd = 2)
+                legend('topright', legend = dist, lty = 1:length(dist),
+                       col = color, ncol = 1, lwd = 2)
             } else if(legend.position == "horizontal") {
-                legend('topright', dist, lty = 1:length(dist), col = color, ncol = 2, lwd = 2)
+                legend('topright', dist, lty = 1:length(dist),
+                       col = color, ncol = 2, lwd = 2)
             }
             if(!is.null(error.rate.sd.concat))
             {
-                col = matrix(color, nrow = nrow(error.rate.concat), ncol = length(dist), byrow=TRUE)
-
-                new_sd.error=error.rate.sd.concat[, grep(mea, colnames(error.rate.sd.concat)), drop = FALSE]
+                col = matrix(color, nrow = nrow(error.rate.concat),
+                             ncol = length(dist), byrow=TRUE)
+                
+                new_sd.error=error.rate.sd.concat[,
+                                                  grep(mea, colnames(error.rate.sd.concat)), drop = FALSE]
                 for(j in 1:ncol(new_mat.error))
-                plot_error_bar(x = component, y = new_mat.error[, j], uiw=new_sd.error[, j], add=T, col = col[, j])
+                    plot_error_bar(x = component, y = new_mat.error[, j],
+                                   uiw=new_sd.error[, j], add=TRUE, col = col[, j])
             }
             title(mea, line = 1)
         }
@@ -184,17 +223,17 @@ plot_error_bar <- function (x, y = as.numeric(x), uiw, add=FALSE, col = "black")
     scols <- col
     
     arrow.args <- c(list(lty = par("lty"), angle = 90, length = smidge,
-    code = 1, col = scols))
+                         code = 1, col = scols))
     
     do.call("arrows", c(list(x[nz], li[nz], x[nz], pmax(y -
-    gap, li)[nz]), arrow.args))
+                                                            gap, li)[nz]), arrow.args))
     
     nz <- abs(ui - pmin(y + gap, ui)) * y.to.in > 0.001
     #scols <- rep(scol, length.out = length(x))[nz]
     arrow.args$col <- scols
     
     do.call("arrows", c(list(x[nz], ui[nz], x[nz], pmin(y +
-    gap, ui)[nz]), arrow.args))
+                                                            gap, ui)[nz]), arrow.args))
     
     do.call("points", c(list(x, y, bg = par("bg"), col =col)))
     
